from typing import Literal
from pathlib import Path
from abc import ABC

from openai import OpenAI, Stream
from openai.resources.chat.completions import Completions
from openai.resources.embeddings import Embeddings
from openai.resources.audio.transcriptions import Transcriptions

#from oaikit.utils import answer_from_stream
#DEFAULT_OPENAI_API_BASE = "https://api.openai.com/v1"
#openai.api_base = os.getenv(OPENAI_API_BASE, DEFAULT_OPENAI_API_BASE)

DEFAULT_MODEL = "gpt-4o"
LANGUAGE_DEFAULT = "es"

class BaseOAI(ABC):
    def __init__(self, *, api_key: str):
        self._client = OpenAI(api_key=api_key)

    @property
    def client(self) -> OpenAI:
        return self._client
    
    @property
    def completions(self) -> Completions:
        return self.client.chat.completions

    @property
    def embeddings(self) -> Embeddings:
        return self.client.embeddings

    @property
    def transcriptions(self) -> Transcriptions:
        return self.client.audio.transcriptions
    
    def transcript(
            self,
            *,
            path_file: Path,
            language: Literal["es", "en"] = LANGUAGE_DEFAULT,
            model: str = "whisper-1"
        ) -> str:
        transcription = self.transcriptions.create(
            model = model,
            file = open(path_file, "rb"),
            language = language
        )
        return transcription.text
    
    # def get_stream(self, *, messages: List[Dict[str, str]], model: str) -> Stream:
    #     return self.completions.create(model=model, messages=messages, stream=True)

    # def answer_from_messages(self, *, messages: List[Dict[str, str]], model: str = DEFAULT_MODEL) -> str:
    #     stream = self.get_stream(messages=messages, model=model)
    #     answer = answer_from_stream(stream=stream)
    #     return answer


class OAI(BaseOAI):
    def __init__(self, *, api_key: str):
        super().__init__(api_key=api_key)



from typing import Literal, List, Dict, Any

from pydantic import BaseModel

from oaikit.func_calls.properties import PropsBase

TYPE_FUNCTION = "function"
T_Function = Literal["function"]

TYPE_OBJECT = "object"
T_Object = Literal["object"]


class FuncParams(BaseModel):
    type: T_Object = TYPE_OBJECT
    properties: Any             # TODO: Referenciar a las properties.
    required: List[str]


class FuncCall(BaseModel):
    name: str
    description: str
    parameters: FuncParams


class Tool(BaseModel):
    type: T_Function = TYPE_FUNCTION
    function: FuncCall

def tool_from_properties(*, description: str, properties: PropsBase) -> Tool:   # TODO: Referenciar a las properties.
    """ Por ahora, hago esta utilidad para generar una tool nueva rápidamente, luego hay que mejorar el circuito."""
    required = list(properties.model_dump().keys())
    parameters = FuncParams(properties=properties, required=required) # TODO: Será siempre deseable el required?
    fn_call = FuncCall(name=properties.fn_name(), description=description, parameters=parameters)
    tool = Tool(function=fn_call)
    return tool
